<?php
/**
 * @author Jegstudio
 * @since 1.0.0
 */

namespace JEG_WEATHER\Elements\Views;

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use JEG_WEATHER\Weather;
use Jeg\Element\Elements\Elements_View_Abstract;

/**
 * Class Weather_View_Abstract
 *
 * @package JEG_WEATHER\Elements\Views
 */
abstract class Weather_View_Abstract extends Elements_View_Abstract {

	/**
	 * @var Weather
	 */
	public $weather_instance;

	/**
	 * Weather_View_Abstract constructor.
	 *
	 * @param $id
	 * @param $classes
	 */
	public function __construct( $id, $classes ) {
		parent::__construct( $id, $classes );
		$this->weather_instance = Weather::get_instance();
	}

	/**
	 * Get weather temp in celcius
	 *
	 * @param $temp
	 *
	 * @return int
	 */
	public function get_weather_temp_celcius( $temp ) {
		return $this->weather_instance->convert_to_celcius( $temp );
	}

	/**
	 * Get weather icon
	 *
	 * @param $condition
	 *
	 * @return array
	 */
	public function get_weather_icon( $condition ) {
		return $this->weather_instance->get_icon_condition_handler( $condition );
	}

	/**
	 * Get section
	 *
	 * @param $section
	 * @param $value
	 *
	 * @return string
	 */
	public function get_section( $section, $value ) {
		$output = '';
		switch ( $section ) {
			case 'location':
				$location = $value;
				$output   = '<div class="jeg_weather_location">
								<span>' . $location . '</span>
							</div>';
				break;
			case 'date':
				$date   = $value;
				$output = '<div class="jeg_weather_date">
								<span>' . date_i18n( $date ) . '</span>
							</div>';
				break;
		}

		return $output;
	}

	/**
	 * Render weather
	 *
	 * @param $attr
	 * @param string $type
	 *
	 * @return string
	 */
	public function render_weather( $attr, $type = 'normal' ) {
		$data         = $this->get_weather_data( $attr );
		$location     = empty( $attr['location_name'] ) && $data ? $data['location']['city'] . ', ' . $data['location']['country'] : $attr['location_name'];
		$icon         = $this->get_weather_icon( $data['now']['condition'] );
		$temp_celcius = $this->get_weather_temp_celcius( $data['now']['temp'] );
		$bg_class     = '';
		$background   = $bg_class;
		$item_count   = is_array( $attr['count'] ) ? $attr['count']['size'] : $attr['count'];
		$style        = isset($attr['style']) ? $attr['style'] : '';
		if ( ! empty( $attr['background'] ) ) {
			$image_id = is_array( $attr['background'] ) ? $attr['background']['id'] : $attr['background'];
			if ( ! empty( $image_id ) ) {
				$background = wp_get_attachment_image_src( $image_id, 'full' );
				$background = $background[0];
				$background = "style=\"background-image: url({$background})\"";
				$bg_class   = 'background';
			}
		}
		$date   = 'inline' !== $type ? $this->get_section( 'date', $attr['date'] ) : '';

		return "<div class=\"jeg_weather_widget $type {$bg_class} {$style}\" {$background}>
                        <div class=\"jeg_weather_now\">
                            <div class=\"jeg_weather_head\">
                                {$this->get_section( 'location', $location )}
                                {$date}
                            </div>
                            <div class=\"jeg_weather_desc\">
                                <div class=\"jeg_weather_today clearfix\">
                                    <div class=\"jeg_weather_icon\">
                                        <span class=\"jeg_weather_{$data['now']['condition']}\">
                                            <i class=\"jegicon {$icon['icon_lg']}\"></i>
                                        </span>
                                        <span class=\"jeg_weather_condition\">{$data['now']['desc']}</span>
                                    </div>
                                    <div class=\"jeg_weather_temp\">
                                        <span class=\"jeg_weather_value\" data-temp-c=\"{$temp_celcius}\" data-temp-f=\"{$data['now']['temp']}\">{$this->weather_instance->convert_temperature($data['now']['temp'])}</span>
                                        <span class=\"jeg_weather_degrees\">&deg;</span>
                                        <span class=\"jeg_weather_unit\" data-unit=\"{$this->weather_instance->temp_unit}\">{$this->weather_instance->temp_unit}</span>
                                    </div>
                                </div>
                                " . $this->build_weather_detail( $attr['detail'], $data ) . "
                            </div>
                        </div>
                        <div class=\"jeg_weather_next {$attr['item']} item_{$item_count} clearfix\">
                            " . $this->build_weather_item( $data, $item_count ) . '
                        </div>
                    </div>';
	}

	/**
	 * Hold weather data
	 *
	 * @param $attr
	 *
	 * @return bool|array
	 */
	public function get_weather_data( $attr ) {
		$weather_data         = false;
		$is_override_location = ! empty( $attr['override_location'] );
		$is_auto_location     = $is_override_location ? ! empty( $attr['auto_location'] ) : jeg_weather_option( 'weather_location_auto', false );
		$location             = $is_override_location ? $attr['weather_location'] : jeg_weather_option( 'global_weather_location', '' );
		$source               = jeg_weather_option( 'weather_forecast_source', 'yahoo' );
		if ( $is_auto_location ) {
			$auto_location = $this->weather_instance->get_location();

			if ( $auto_location ) {
				$location              = $auto_location;
				$attr['location_name'] = null;
			}
		}
		if ( ! empty( $location ) ) {
			$data = $this->weather_instance->check_cache( $source, $location );
			if ( is_array( $data ) && ! empty( $data ) ) {
				$weather_data = $data;
			}
		}

		return $weather_data;
	}

	/**
	 * Build weather item content
	 *
	 * @param array $data
	 * @param int   $count
	 *
	 * @return string
	 */
	public function build_weather_item( $data, $count ) {
		$output  = '';
		$counter = 0;

		if ( ! is_array( $data['next'] ) ) {
			return false;
		}

		foreach ( $data['next'] as $item ) {
			$counter ++;

			if ( $counter > $count ) {
				break;
			}

			$icon = $this->get_weather_icon( $item['condition'] );

			$output .= "<div class=\"jeg_weather_item\">
                            <div class=\"jeg_weather_icon\">
                                <span class=\"jeg_weather_{$data['now']['condition']}\">
                                    <i class=\"jegicon {$icon['icon_sm']}\"></i>
                                </span>
                            </div>
                            <div class=\"jeg_weather_temp\">
                                <span class=\"max\"><span class=\"jeg_weather_value\" data-temp-c=\"{$this->get_weather_temp_celcius($item['temp_max'])}\" data-temp-f=\"{$item['temp_max']}\">{$this->weather_instance->convert_temperature($item['temp_max'])}</span>
                                    <span class=\"jeg_weather_unit\" data-unit=\"{$this->weather_instance->temp_unit}\">{$this->weather_instance->temp_unit}</span>
                                </span>
                                <span class=\"jeg_weather_separator\"></span>
                                <span class=\"min\"><span class=\"jeg_weather_value\" data-temp-c=\"{$this->get_weather_temp_celcius($item['temp_min'])}\" data-temp-f=\"{$item['temp_min']}\">{$this->weather_instance->convert_temperature($item['temp_min'])}</span>
                                    <span class=\"jeg_weather_unit\" data-unit=\"{$this->weather_instance->temp_unit}\">{$this->weather_instance->temp_unit}</span>
                                </span>
                            </div>
                            <div class=\"jeg_weather_day\">
                                <span class=\"jeg_weather_day\">" . date_i18n( 'D', $item['time'] ) . '</span>
                            </div>
                        </div>';

		}

		return $output;
	}

	/**
	 * Build weather detail content
	 *
	 * @param string $detail
	 * @param array  $data
	 *
	 * @return string
	 */
	public function build_weather_detail( $detail, $data ) {
		$output = '';

		if ( 'show' === $detail ) {
			/**
			 * Some weather provider already provide percent, replace them to avoid duplicate character
			 *
			 * @var $clouds
			 */
			$clouds = str_replace( '%%', '%', $data['now']['clouds'] . '%' );
			$output = "<div class=\"jeg_weather_detail clearfix\">
                            <div class=\"jeg_weather_humidity\">
                                <i class=\"fa fa-tint\"></i>
                                <span class=\"jeg_weather_value\">{$data['now']['humidity']}%</span>
                            </div>
                            <div class=\"jeg_weather_wind\">
                                <i class=\"jegicon jegicon-windy-sm\"></i>
                                <span class=\"jeg_weather_value\">{$data['now']['speed']}mh</span>
                            </div>
                            <div class=\"jeg_weather_cloud\">
                                <i class=\"fa fa-cloud\"></i>
                                <span class=\"jeg_weather_value\">{$clouds}</span>
                            </div>
                        </div>";
		}

		return $output;
	}

	/**
	 * @param $result
	 * @param $attr
	 *
	 * @return string
	 */
	public function render_result_element( $result, $attr ) {
		return '';

	}
}
